package zero1hd.rhythmbullet.audio.metadata;

import java.io.IOException;

import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.audio.exceptions.CannotReadException;
import org.jaudiotagger.audio.exceptions.CannotWriteException;
import org.jaudiotagger.audio.exceptions.InvalidAudioFrameException;
import org.jaudiotagger.audio.exceptions.ReadOnlyFileException;
import org.jaudiotagger.audio.mp3.MP3File;
import org.jaudiotagger.tag.TagException;
import org.jaudiotagger.tag.id3.ID3v23FieldKey;
import org.jaudiotagger.tag.id3.ID3v23Tag;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;

public class MP3Metadata implements AudioMetadata {
	private String title, author, duration, genre;
	private int length;
	private Texture albumCover;
	private FileHandle fileHandle;
	
	public MP3Metadata(FileHandle fileHandle) {
		this.fileHandle = fileHandle;
		
		try {
			MP3File mp3file = (MP3File) AudioFileIO.read(fileHandle.file());
			ID3v23Tag tag;
			if (mp3file.hasID3v1Tag()) {
				mp3file.setID3v2Tag(mp3file.getID3v1Tag());
				mp3file.setID3v1Tag(null);
				mp3file.commit();
				tag = (ID3v23Tag) mp3file.getTagAndConvertOrCreateAndSetDefault();
			}
			tag = (ID3v23Tag) mp3file.getTagAndConvertOrCreateAndSetDefault();

				
			
			length = mp3file.getAudioHeader().getTrackLength();
			int min = (int) (length/60);
			duration = (length/60) + ":" + (length - (min*60));
			
			author = tag.getFirst(ID3v23FieldKey.ARTIST);
			genre = tag.getFirst(ID3v23FieldKey.GENRE);
			title = tag.getFirst(ID3v23FieldKey.TITLE);
			
		} catch (IOException | CannotWriteException | CannotReadException | TagException | ReadOnlyFileException | InvalidAudioFrameException e) {
			Gdx.app.error("MP3Metadata", "Failed to read metadata of file: " + fileHandle.name());
		}
	}
	
	@Override
	public void loadAlbumCover() {
		MP3File mp3file;
		try {
			mp3file = (MP3File) AudioFileIO.read(fileHandle.file());
			
			byte[] imageData = mp3file.getTag().getFirstArtwork().getBinaryData();
			Pixmap pixmap = new Pixmap(imageData, 0, imageData.length);
			albumCover = new Texture(pixmap);
			pixmap.dispose();
			
		} catch (CannotReadException | IOException | TagException | ReadOnlyFileException | InvalidAudioFrameException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void unloadAlbumCover() {
		if (albumCover != null) {
			albumCover.dispose();
			albumCover = null;
		}
	}

	@Override
	public String getAuthor() {
		return author;
	}

	@Override
	public String getTitle() {
		return title;
	}

	@Override
	public String getDuration() {
		return duration;
	}
	
	@Override
	public int getLength() {
		return length;
	}
	
	@Override
	public String getGenre() {
		return genre;
	}
	
	@Override
	public Texture getAlbumCover() {
		return albumCover;
	}

	@Override
	public FileHandle getFileHandle() {
		return fileHandle;
	}
	
	@Override
	public void dispose() {
		unloadAlbumCover();
	}


}
