package zero1hd.rhythmbullet.graphics.ui.components;

import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.ui.Button;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Array;

import zero1hd.rhythmbullet.audio.metadata.AudioMetadata;

public class MusicSelectable extends Button {
	private Vector2 actualCoords;
	private Image album;
	private Table informationTable;
	private ShortenedLabel name, artist;
	private Label time;
	private float timeSinceOnScreen;
	private AudioMetadata metadata;
	private Texture defaultAlbumArt;
	private Array<MusicSelectable> queueList;

	public MusicSelectable(Skin skin, Texture defaultAlbumArt, AudioMetadata metadata,
			Array<MusicSelectable> queueList) {
		super(skin, "music-selectable");
		this.metadata = metadata;
		this.defaultAlbumArt = defaultAlbumArt;
		this.queueList = queueList;
		
		album = new Image(defaultAlbumArt);
		add(album).expand().left();
		informationTable = new Table();
		name = new ShortenedLabel(metadata.getTitle(), skin, "default-font", skin.getColor("default"));
		informationTable.add(name).colspan(2).expandX();
		informationTable.row();
		artist = new ShortenedLabel(metadata.getTitle(), skin, "sub-font", skin.getColor("default"));
		informationTable.add(artist).expandX();
		time = new Label(metadata.getDuration(), skin, "sub-font", skin.getColor("default"));
		informationTable.add(time).expandX();
		add(informationTable).expand().fill();
	}

	@Override
	public void act(float delta) {
		actualCoords.x = getX() + getParent().getX();
		actualCoords.y = getY() + getParent().getY();

		if (actualCoords.y < 0 - getHeight() || actualCoords.y > getStage().getHeight()
				|| actualCoords.x < 0 - getWidth() || actualCoords.x > getStage().getWidth()) {
			offScreenAct(delta);
		} else {
			onScreenAct(delta);
		}
		super.act(delta);
	}

	@Override
	public void draw(Batch batch, float parentAlpha) {
		synchronized (album) {
			super.draw(batch, parentAlpha);
		}
	}
	
	public void onScreenAct(float delta) {
		timeSinceOnScreen = 0;
		if (!queueList.contains(this, true)) {
			synchronized (queueList) {
				queueList.add(this);
				notify();
			}
		}
	}

	public void offScreenAct(float delta) {
		if (metadata.getAlbumCover() != null) {
			timeSinceOnScreen += delta;
			if (timeSinceOnScreen >= 2) {
				album.setDrawable(new TextureRegionDrawable(new TextureRegion(defaultAlbumArt)));
				metadata.unloadAlbumCover();
			}
		}
	}

	public void loadAlbumCover() {
		metadata.loadAlbumCover();
		Texture texture = defaultAlbumArt;
		if (metadata.getAlbumCover() != null) {
			texture = metadata.getAlbumCover();
		}
		synchronized (album) {
			album.setDrawable(new TextureRegionDrawable(new TextureRegion(texture)));
		}
	}

	public AudioMetadata getMetadata() {
		return metadata;
	}
	
	public FileHandle getFileHandle() {
		return metadata.getFileHandle();
	}
}
