package zero1hd.rhythmbullet.graphics.ui.components;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.GlyphLayout;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;

public class ShortenedLabel extends Label {
	private String originalText;
	private int targetWidth;
	private GlyphLayout gl;
	private BitmapFont font;
	private Vector2 size;
	
	public ShortenedLabel(CharSequence text, Skin skin, String fontName, Color color) {
		super(null, skin, fontName, color);
		originalText = text.toString();
		font = skin.getFont(fontName);
		if (text != null) {
			gl = new GlyphLayout(skin.getFont(fontName), text);
		}
		size = new Vector2();
		
		setWrap(true);
	}
	
	public void resize() {
		setToOriginalText();
		String text = getText().toString();
		while (gl.width > targetWidth && (text.length() - 4) > 0) {
			text = text.substring(0, text.length() - 4).concat("...");
			gl.setText(font, text);
		}
		setText(text);
	}
	
	public void setToOriginalText() {
		setText(originalText);
		gl.setText(font, originalText);
	}
	
	@Override
	public void layout() {
		super.layout();
		size.x = getWidth();
		size.y = getHeight();
		targetWidth = (int) getStage().stageToScreenCoordinates(size).x;
		resize();
	}
	
	public void setOriginalText(String originalText) {
		this.originalText = originalText;
		gl.setText(font, originalText);
	}
	
	public int getTargetWidth() {
		return targetWidth;
	}
}
