package zero1hd.rhythmbullet.desktop.graphics.ui.components;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.FocusListener;
import com.badlogic.gdx.utils.Align;

import zero1hd.rhythmbullet.controls.KeyMap;

public class ControlOptions extends Table {
	private KeyBindButton selected;
	
	public ControlOptions(Skin skin, KeyMap keyMap) {
		super(skin);
		align(Align.center);
		
		ClickListener selectedListener = new ClickListener() {
			@Override
			public void clicked(InputEvent event, float x, float y) {
				getStage().setKeyboardFocus(event.getTarget());
				event.getTarget().setColor(Color.ORANGE);
				selected = (KeyBindButton) event.getTarget();
				super.clicked(event, x, y);
			}
		};
		
		InputListener keyPressedListener = new InputListener() {
			@Override
			public boolean keyUp(InputEvent event, int keycode) {
				getStage().setKeyboardFocus(null);
				Gdx.app.debug("KeySetter", "input keycode received: " + keycode);
				if (keycode != Keys.ESCAPE) {
					Gdx.app.debug("keySetter", "input has been set to: " + Keys.toString(keycode));
					if (selected.setKey(keycode)) {
						keyMap.updateKeys();
						selected.addAction(Actions.sequence(Actions.color(Color.GREEN, 0.2f), Actions.color(Color.WHITE, 0.2f)));
					} else {
						selected.addAction(Actions.sequence(Actions.color(Color.RED, 0.2f), Actions.delay(0.1f), Actions.color(Color.WHITE, 0.2f)));
					}
				}
				return super.keyUp(event, keycode);
			}
		};
		
		FocusListener deselected = new FocusListener() {
			@Override
			public void keyboardFocusChanged(FocusEvent event, Actor actor, boolean focused) {
				if (!focused) {
					unselect();
				}
				super.keyboardFocusChanged(event, actor, focused);
			}
		};
		
		Label forwardKeyLabel = new Label("Forward: ",skin);
		forwardKeyLabel.setName(KeyMap.UP);
		add(forwardKeyLabel).left();
		KeyBindButton forwardKeySetter = new KeyBindButton(keyMap, KeyMap.UP, selectedListener, keyPressedListener, deselected);
		add(forwardKeySetter).spaceRight(45f);
		Label shootKeyLabel = new Label("Shoot: ", skin);
		shootKeyLabel.setName(KeyMap.SHOOT);
		add(shootKeyLabel).left();
		KeyBindButton shootKeySetter = new KeyBindButton(keyMap, KeyMap.SHOOT, selectedListener, keyPressedListener, deselected);
		add(shootKeySetter);
		
		row();

		Label backwardKeyLabel = new Label("Backward: ", skin);
		backwardKeyLabel.setName(KeyMap.DOWN);
		add(backwardKeyLabel).left();
		KeyBindButton backwardKeySetter = new KeyBindButton(keyMap, KeyMap.DOWN, selectedListener, keyPressedListener, deselected);
		add(backwardKeySetter).spaceRight(45f);
		Label sector1TPKeyLabel = new Label("Left Teleport", skin);
		sector1TPKeyLabel.setName(KeyMap.FIRSTTHIRDTELEPORT);
		add(sector1TPKeyLabel).left();
		KeyBindButton Sector1TPKeySetter = new KeyBindButton(keyMap, KeyMap.FIRSTTHIRDTELEPORT, selectedListener, keyPressedListener, deselected);
		Sector1TPKeySetter.addListener(selectedListener);
		add(Sector1TPKeySetter);
		
		row();
		
		Label leftKeyLabel = new Label("Left: ", skin);
		leftKeyLabel.setName(KeyMap.LEFT);
		add(leftKeyLabel).left();
		KeyBindButton leftKeySetter = new KeyBindButton(keyMap, KeyMap.LEFT, selectedListener, keyPressedListener, deselected);
		add(leftKeySetter).spaceRight(45f);
		Label sector2TPKeyLabel = new Label("Middle Teleport: ", skin);
		sector2TPKeyLabel.setName(KeyMap.SECONDTHIRDTELEPORT);
		add(sector2TPKeyLabel).left();
		KeyBindButton sector2TPKeySetter = new KeyBindButton(keyMap, KeyMap.SECONDTHIRDTELEPORT, selectedListener, keyPressedListener, deselected);
		sector2TPKeySetter.addListener(selectedListener);
		add(sector2TPKeySetter);
		
		row();
		
		Label rightKeyLabel = new Label("Right: ", skin);
		rightKeyLabel.setName(KeyMap.RIGHT);
		add(rightKeyLabel).left();
		KeyBindButton rightKeySetter = new KeyBindButton(keyMap, KeyMap.RIGHT, selectedListener, keyPressedListener, deselected);
		add(rightKeySetter).spaceRight(45f);
		Label sector3TPKeyLabel = new Label("Right Teleport: ", skin);
		sector3TPKeyLabel.setName(KeyMap.THIRDTHIRDTELEPORT);
		add(sector3TPKeyLabel).left();
		KeyBindButton sector3TPKeySetter = new KeyBindButton(keyMap, KeyMap.THIRDTHIRDTELEPORT, selectedListener, keyPressedListener, deselected);
		sector3TPKeySetter.addListener(selectedListener);
		add(sector3TPKeySetter);
		
		setFillParent(true);
	}
	
	private void unselect() {
		selected.setColor(Color.WHITE);
	}
}
