package zero1hd.rhythmbullet.desktop.screens.main;

import java.util.Observable;
import java.util.Observer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Array;

import zero1hd.rhythmbullet.audio.MusicMetadataController;
import zero1hd.rhythmbullet.audio.MusicController;
import zero1hd.rhythmbullet.graphics.ui.Page;
import zero1hd.rhythmbullet.graphics.ui.components.MusicSelectable;
import zero1hd.rhythmbullet.graphics.ui.components.MusicSelectableButtonGroup;
import zero1hd.rhythmbullet.graphics.ui.components.ScrollText;

public class MusicSelectionPage extends Page implements Observer {
	Preferences musicFileAnnotation;
	
	private MusicController mc;
	private MusicMetadataController mmc;
	private MusicSelectableButtonGroup selectables;
	private Stack stackSelectables;
	private TextButton back;
	private ScrollPane musicTableScrollPane;
	
	private ClickListener selectionListener;
	private musicSelectionLoaderThread thread;
	
	private Table musicInfoTable;
	private Table musicSubInfo;
	private ScrollText songTitle;
	private Label author;
	private Label songLength;
	private Label previousTop;
	private Label ratedDifficulty;
	private Image albumCover;
	
	private AssetManager assets;
	private Skin skin;
	
	private boolean down, up;
	
	private TextButton beginButton;
	
	private float scrollTimer, scrollDelay = 0.2f, scrollDelMod, songSelectionTimer;
	
	public MusicSelectionPage(AssetManager assetManager, Skin skin, MusicController musicController, MusicMetadataController musicMetadataController, ChangeListener backButtonListener, ChangeListener beginButtonListener) {
		super(1, 0);
		this.assets = assetManager;
		this.mc = musicController;
		this.mmc = musicMetadataController;
		this.skin = skin;
		stackSelectables = new Stack();
		selectables = new MusicSelectableButtonGroup();
		musicFileAnnotation = Gdx.app.getPreferences("music_file_annotation");
		
		musicTableScrollPane = new ScrollPane(stackSelectables, skin);
		musicTableScrollPane.setSize(0.45f*getWidth(), getHeight());
		musicTableScrollPane.setFadeScrollBars(false);
		musicTableScrollPane.setOverscroll(false, false);
		musicTableScrollPane.setColor(Color.BLUE);
		addActor(musicTableScrollPane);
		back = new TextButton("Back", skin);
		back.setWidth(back.getWidth()+20f);
		back.setPosition(getWidth()-back.getWidth()-15f, getHeight() - back.getHeight() - 15f);
		back.addListener(backButtonListener);
		addActor(back);
		back.toFront();
		
		addListener(new InputListener() {
			@Override
			public boolean keyDown(InputEvent event, int keycode) {
				scrollTimer = 0;
				scrollDelMod = 1f;
				if (keycode == Keys.DOWN) {
					down = true;
				}
				if (keycode == Keys.UP) {
					up = true;
				}
				return super.keyDown(event, keycode);
			}
			
			@Override
			public boolean keyUp(InputEvent event, int keycode) {
				if (keycode == Keys.DOWN) {
					down = false;
				}
				
				if (keycode == Keys.UP) {
					up = false;
				}
				return super.keyUp(event, keycode);
			}
		});
		
		musicInfoTable = new Table();
		musicInfoTable.defaults().center();
		musicInfoTable.setPosition(musicTableScrollPane.getWidth() + musicTableScrollPane.getX(), 0);
		musicInfoTable.setSize(getWidth()-musicTableScrollPane.getWidth(), getHeight());
		addActor(musicInfoTable);
		musicSubInfo = new Table(skin);
		musicSubInfo.setBackground("corner-panel");
		songTitle = new ScrollText("", null, skin, true, true);
		author = new Label(null, skin, "sub-font", skin.getColor("default"));
		songLength = new Label(null, skin, "sub-font", skin.getColor("default"));
		previousTop = new Label(null, skin, "sub-font", skin.getColor("default"));
		ratedDifficulty = new Label(null, skin, "sub-font", skin.getColor("default"));
		albumCover = new Image(assets.get("defaultCover.png", Texture.class));
		
		beginButton = new TextButton("Begin", skin);
		beginButton.addListener(beginButtonListener);
		
		mmc.addObserver(this);
		
		thread = new musicSelectionLoaderThread();
		
		selectionListener  = new ClickListener() {
			public void clicked(InputEvent event, float x, float y) {
				MusicSelectable selectable = (MusicSelectable) event.getListenerActor();
				if (selectable.getMetadata().getTitle() != null) {
					songTitle.setText(selectable.getMetadata().getTitle(), null);
				} else {
					songTitle.setText(selectable.getFileHandle().nameWithoutExtension(), null);
				}
				
				author.setText(selectable.getMetadata().getAuthor());
				
				songLength.setText(selectable.getMetadata().getDuration());
				
				previousTop.setText("...");
				
				ratedDifficulty.setText("...");
				
				if (selectable.getMetadata().getAlbumCover() != null) {
					albumCover.setDrawable(new TextureRegionDrawable(new TextureRegion(selectable.getMetadata().getAlbumCover())));
				} else {
					albumCover.setDrawable(new TextureRegionDrawable(new TextureRegion(assets.get("defaultCover.png", Texture.class))));
				}
			}
		};
	}
	
	@Override
	public void act(float delta) {
		if (down) {
			if (scrollDelMod > 0.25f) {
				scrollDelMod -= delta/0.5f;
			}
			if (scrollTimer <= 0) {
				scrollTimer = scrollDelay*scrollDelMod;
				scrollDown();

			} else {
				scrollTimer -= delta;
			}
		}
		
		if (up) {
			if (scrollDelMod > 0.25f) {
				scrollDelMod -= delta/0.5f;
			}
			if (scrollTimer <= 0) {
				scrollTimer = scrollDelay*scrollDelMod;
				scrollUp();
			} else {
				scrollTimer -= delta;
			}
		}
		
		if (songSelectionTimer > 0f) {
			songSelectionTimer -= delta;
			if (songSelectionTimer <= 0f) {
			}
		}
		
		if (mc.getMusicList().isSearched() && selectables.getButtons().size == mc.getMusicList().getTotal()) {
			if (selectables.getButtons().size != stackSelectables.getChildren().size) {
				int index = selectables.getButtons().size - stackSelectables.getChildren().size - 1;
				stackSelectables.add(selectables.getButtons().get(index));
			} else {
				if (selectables.getChecked() == null) {
					selectables.setChecked(mc.getCurrentMusicFileHandle());
				} else if (selectables.getChecked().getFileHandle() != mc.getCurrentMusicFileHandle()) {
					songSelectionTimer += delta;
					if (songSelectionTimer > 2f) {
						mc.setMusicByFileHandle(selectables.getChecked().getFileHandle());
					}
				} else {
					songSelectionTimer = 0;
				}
			}
		}
		super.act(delta);
	}
	
	private void scrollDown() {
		selectables.selectNext();
		musicTableScrollPane.scrollTo(selectables.getChecked().getX(), selectables.getChecked().getY(), selectables.getChecked().getWidth(), selectables.getChecked().getHeight());
	}
	
	private void scrollUp() {
		selectables.selectPrevious();
		musicTableScrollPane.scrollTo(selectables.getChecked().getX(), selectables.getChecked().getY(), selectables.getChecked().getWidth(), selectables.getChecked().getHeight());
	}
	
	public FileHandle getSelectedMusic() {
		return selectables.getChecked().getMetadata().getFileHandle();
	}
	
	public void refreshUIList() {
		selectables.clear();
		
		mmc.loadAudioMetadata();
		selectables.clear();
		musicInfoTable.clear();
		musicSubInfo.clear();
		
		Gdx.app.debug("MusicSelectionPage", "Refreshing...");
		
		songTitle.setText("loading...", null);
		musicInfoTable.add(songTitle).width(musicInfoTable.getWidth()*0.6f).spaceBottom(30f);
		musicInfoTable.row();
		author.setText("...");
		musicSubInfo.add(author);
		musicSubInfo.row();
		songLength.setText("...");
		musicSubInfo.add(songLength);
		musicSubInfo.row();
		previousTop.setText("...");
		musicSubInfo.add(previousTop);
		musicSubInfo.row();
		ratedDifficulty.setText("...");
		musicSubInfo.add(ratedDifficulty);
		musicSubInfo.pack();
		musicInfoTable.add(musicSubInfo).spaceBottom(20f);
		musicInfoTable.row();
		albumCover.setDrawable(new TextureRegionDrawable(new TextureRegion(assets.get("defaultCover.png", Texture.class))));
		musicInfoTable.add(albumCover).size(musicInfoTable.getWidth()/2f);
		musicInfoTable.row();
		musicInfoTable.add(beginButton).spaceTop(20f).fillX();
	}
	
	@Override
	public void dispose() {
		super.dispose();
	}
	
	@Override
	public void update(Observable o, Object arg) {
		if (o == mmc) {
			thread.start();
		}
	}
	
	@Override
	public void setCameraPositionToPage(Vector3 cameraPosition) {
		getStage().setKeyboardFocus(this);
		super.setCameraPositionToPage(cameraPosition);
	}
	
	private class musicSelectionLoaderThread implements Runnable {
		private Thread thread;
		private Array<MusicSelectable> queueList;
		private String name = "Music-Selection-Loader-Thread";
		private volatile boolean work;
		
		public musicSelectionLoaderThread() {
			queueList = new Array<>();
		}
		
		@Override
		public void run() {
			while (work) {
				while (selectables.getButtons().size != mc.getMusicList().getTotal()) {
					MusicSelectable selectable = new MusicSelectable(skin, assets.get("defaultCover.png"), mmc.getMetadata(selectables.getButtons().size), queueList);
					selectable.addListener(selectionListener);
					selectables.add(selectable);
				}
				
				for (int i = 0; i < queueList.size; i++) {
					queueList.get(i).loadAlbumCover();
					queueList.removeIndex(i);
				}
				
				synchronized (queueList) {
					try {
						wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		public boolean start() {
			if (thread == null) {
				thread = new Thread(this, name);
				thread.start();
				return true;
			}
			synchronized (queueList) {
				notify();
			}
			return false;
		}
		
	}
}
